// Delcom Products Inc
// DelcomDLL.h - Internal DLL header file.
// 02/06/03

#pragma once
       

// --------------------------------------------------------- //
// DEFINES
// --------------------------------------------------------- //
#define MaxDeviceLen 512
		
#define GREENLED	0
#define	REDLED		1
#define BLUELED		2


#define DRV_TYPE_NONE	0
#define DRV_TYPE_HID	1
#define DRV_TYPE_DELCOM	2



// The following constants determine which HID device to find and connect to.
#define USB_HID_VID 0x0FC5		// USB Vendor ID (Always 0x0FC5 for Delcom products)
#define USB_HID_PID 0xB080		// USB Product ID (Always 0xB080 for Delcom HID device)
#define USB_HID_TID 0x0001		// USB Type ID	(1 for Delcom USBHIDIO device, zero=scan for all)
#define USB_HID_SID	0x0000		// USB Serial ID (zero=scan for all)


// --------------------------------------------------------- //
// TYPEDEF
// --------------------------------------------------------- //
typedef unsigned char uchar; 


typedef struct _PacketStruct{
		uchar Recipient;			// byte0
		uchar DeviceModel;			// byte1
		uchar MajorCmd;				// byte2
		uchar MinorCmd;				// byte3
		uchar DataLSB;				// byte4
		uchar DataMSB;				// byte5
		SHORT Length;				// byte6
		uchar ExtData[8];
	} PacketStruct,*pPacketStruct;



typedef struct _DeviceNameStruct{
	char DeviceName[MaxDeviceLen];
} DeviceNameStruct, *pDeviceNameStruct;

typedef struct MyDeviceStruct_ {
	HANDLE Handle;
	UINT DType;
	char Name[MaxDeviceLen];		// currently 512
	} MyDeviceStruct, *pMyDeviceStruct;



// HID Packet DataStruct
typedef union HIDPacketStruct { 
	char Data[256];
	struct {
		char MajorCmd;
		char MinorCmd;
		char DataLSB;
		char DataMSB;
		char DataHid[4];
		char DataExt[8];
		} Tx;
	struct {
		char Cmd;
		} Rx;

}	HIDPacketStruct, *pHIDPacketStruct; 

// DataStruct used by the GetDeviceInfo functions
typedef struct DelcomDeviceInfoStruct_ {
		unsigned short int Family;
		unsigned short int Security;
		unsigned char Version;
		unsigned char Day;
		unsigned char Month;
		unsigned char Year;
		unsigned int Serial;
		unsigned int Spare;
		} DelcomDeviceInfoStruct, *pDelcomDeviceInfoStruct;





// --------------------------------------------------------- //
// Private DLL Function Prototyping
// --------------------------------------------------------- //
HANDLE IsDeviceOpen(char* );
int IsHandleOpen(HANDLE hUsb);
DWORD CheckHandle(HANDLE hUsb );


// --------------------------------------------------------- //
// EXPORT PUBLIC DLL FUNCTIONS
// --------------------------------------------------------- //


//EXPORTED PUBLIC DLL functions. NOTE must be __stdcall for VB compatablity
__declspec(dllexport) HANDLE __stdcall DelcomGetDLLHandle(void);
__declspec(dllexport) double __stdcall DelcomGetDLLVersion(void);
__declspec(dllexport) DWORD __stdcall DelcomGetDLLDate(LPSTR);
__declspec(dllexport) DWORD __stdcall DelcomVerboseControl( DWORD Mode, LPSTR Header );

__declspec(dllexport) DWORD __stdcall DelcomGetNthDevice(DWORD, DWORD, LPSTR);
__declspec(dllexport) DWORD __stdcall DelcomScanDevices( DWORD, DeviceNameStruct[], DWORD );
__declspec(dllexport) HANDLE __stdcall DelcomOpenDevice( LPSTR, DWORD);
__declspec(dllexport) DWORD __stdcall DelcomCloseDevice(HANDLE);
__declspec(dllexport) DWORD __stdcall DelcomGetDeviceCount( DWORD Type );

__declspec(dllexport) DWORD __stdcall DelcomReadDeviceVersion(HANDLE  );
__declspec(dllexport) DWORD __stdcall DelcomReadDeviceSerialNum(LPSTR, HANDLE );
__declspec(dllexport) DWORD __stdcall DelcomSendPacket( HANDLE, pPacketStruct, pPacketStruct); 
__declspec(dllexport) DWORD __stdcall DelcomSendHIDPacket(HANDLE hUsb, pHIDPacketStruct pHidPacket, DWORD len);
	

// Visual Indicator Functions
__declspec(dllexport) DWORD __stdcall DelcomLEDControl( HANDLE hFile, DWORD Color, DWORD Mode );
__declspec(dllexport) DWORD __stdcall DelcomEnableAutoConfirm(HANDLE hUsb, DWORD Mode);
__declspec(dllexport) DWORD __stdcall DelcomEnableAutoClear(HANDLE hUsb, DWORD Mode);
__declspec(dllexport) DWORD __stdcall DelcomBuzzer(HANDLE hUsb, uchar Mode, uchar Freq, uchar Repeat, uchar OnTime, uchar OffTime);
__declspec(dllexport) DWORD __stdcall DelcomLoadInitialPhaseDelay(HANDLE hUsb, BYTE Color, BYTE Delay);
__declspec(dllexport) DWORD __stdcall DelcomSyncLeds(HANDLE hUsb);
__declspec(dllexport) DWORD __stdcall DelcomLoadPreScalar(HANDLE hUsb, BYTE PreScalar);
__declspec(dllexport) DWORD __stdcall DelcomLoadLedFreqDuty(HANDLE hUsb, BYTE Color, BYTE Low, BYTE High);
__declspec(dllexport) DWORD __stdcall DelcomGetButtonStatus(HANDLE hUsb );
__declspec(dllexport) DWORD __stdcall DelcomLEDPower( HANDLE hUsb, DWORD Color, DWORD Mode );


// USBIODS Functions
__declspec(dllexport) DWORD __stdcall DelcomWritePin(HANDLE hUsb, BYTE Port, BYTE Pin, BYTE Value );
__declspec(dllexport) DWORD __stdcall DelcomWritePorts(HANDLE hUsb, BYTE Port0, BYTE Port1 );
__declspec(dllexport) DWORD __stdcall DelcomSetupPorts(HANDLE hUsb, BYTE Port, BYTE Mode0, BYTE Mode1  );
__declspec(dllexport) DWORD __stdcall DelcomReadPorts(HANDLE hUsb, BYTE* Port0, BYTE* Port1 );

__declspec(dllexport) DWORD __stdcall DelcomWrite64Bit(HANDLE hUsb, LPSTR DataExt );
__declspec(dllexport) DWORD __stdcall DelcomRead64Bit(HANDLE hUsb, LPSTR DataExt );

__declspec(dllexport) DWORD __stdcall DelcomWriteI2C(HANDLE hUsb, BYTE CmdAdd, BYTE Length, LPSTR DataExt );
__declspec(dllexport) DWORD __stdcall DelcomReadI2C(HANDLE hUsb, BYTE CmdAdd, BYTE Length, LPSTR DataExt );
__declspec(dllexport) DWORD __stdcall DelcomSelReadI2C(HANDLE hUsb, BYTE SetAddCmd, BYTE Address, BYTE ReadCmd, BYTE Length, LPSTR DataExt );
__declspec(dllexport) DWORD __stdcall DelcomWriteI2CEEPROM(HANDLE hUsb, DWORD Address, DWORD Length, BYTE CtrlCode, BYTE WriteDelay, LPSTR pData );
__declspec(dllexport) DWORD __stdcall DelcomReadI2CEEPROM(HANDLE hUsb, DWORD Address, DWORD Length, BYTE CtrlCode, LPSTR pData );
																		

__declspec(dllexport) DWORD __stdcall DelcomRS232Ctrl(HANDLE hUsb, DWORD Mode, DWORD Value );
__declspec(dllexport) DWORD __stdcall DelcomWriteRS232(HANDLE hUsb, DWORD Length, LPSTR DataExt );
__declspec(dllexport) DWORD __stdcall DelcomReadRS232(HANDLE hUsb, LPSTR DataExt );


	// SPI Commands 
__declspec(dllexport) DWORD __stdcall DelcomSPIWrite( HANDLE hUsb, DWORD ClockCount, LPSTR pData);
__declspec(dllexport) DWORD __stdcall DelcomSPISetClock( HANDLE hUsb, DWORD ClockPeriod );
__declspec(dllexport) DWORD __stdcall DelcomSPIRead(HANDLE hUsb,  LPSTR pData);
__declspec(dllexport) DWORD __stdcall DelcomSPIWr8Read64( HANDLE hUsb, DWORD WrData, DWORD ClockCount, LPSTR pData);


// USBNDSPY Functions
__declspec(dllexport) DWORD __stdcall DelcomNumericMode(HANDLE hUsb, BYTE Mode, BYTE Rate );
__declspec(dllexport) DWORD __stdcall DelcomNumericScanRate(HANDLE hUsb, BYTE ScanRate );
__declspec(dllexport) DWORD __stdcall DelcomNumericSetup(HANDLE hUsb, BYTE Digits );
__declspec(dllexport) DWORD __stdcall DelcomNumericRaw(HANDLE hUsb, LPSTR Str );
__declspec(dllexport) DWORD __stdcall DelcomNumericInteger(HANDLE hUsb, DWORD Number, DWORD Base );
__declspec(dllexport) DWORD __stdcall DelcomNumericHexaDecimal(HANDLE hUsb, DWORD Number, DWORD Base );
__declspec(dllexport) DWORD __stdcall DelcomNumericDouble(HANDLE hUsb, double Number, DWORD Base );



// --------------------------------------------------------- //
// CLASS HEADER DEFINATIONS
// --------------------------------------------------------- //


// --------------------------------------------------------- //
// CUSBHIDIO CLASS
// --------------------------------------------------------- //
class CUSBHIDIO
{
	protected:
		HIDPacketStruct myPacket;

	public:
	
	CUSBHIDIO();			// Construction
	unsigned int Close(HANDLE hDevice);
	HANDLE Open(char* DeviceName);
	UINT	Found;		// Set in ScanForHidDevice, to the number of match device found
	int ReadCmd(HANDLE hDevice, pHIDPacketStruct pPacket, unsigned int Length);
	int WriteCmd(HANDLE hDevice, pHIDPacketStruct pPacket, unsigned int Length);
	int GetDeviceInfo(HANDLE hDevice, pDelcomDeviceInfoStruct pInfo);
	int GetDeviceCount(int Type);
	int GetNthDevice( int Type, int NthDevice, LPSTR DeviceName);

	int ReadPorts(HANDLE hDevice, unsigned int *pPorts);
	int SendHIDPacket(HANDLE hUsb, pHIDPacketStruct pHidPacket, DWORD len);

	UINT	ScanForHIDDevice(DeviceNameStruct DeviceNames[], LPSTR DeviceName, UINT VID, UINT PID, UINT TID, UINT SID, UINT NthDevice, UINT MaxSize );
	int		Scan(int Type, DeviceNameStruct DeviceNames[] , int MaxCount);

	
	
	


public:
	int Translator(HANDLE hUsb, pPacketStruct pTx, pPacketStruct pRx);
	int FillTestPackets( pHIDPacketStruct pP1, pHIDPacketStruct pP2);
	int IsPacketsMatch(pHIDPacketStruct pPacket1, pHIDPacketStruct pPacket2);
};


// --------------------------------------------------------- //
// CUSBCODE CLASS
// --------------------------------------------------------- //


class CUsbCode
{
	char	msg[512];
	//HANDLE	hUsb;
	
public:
	MyDeviceStruct MyDevice;
	PacketStruct Packet;
	HIDPacketStruct HidPacket;
	BYTE ReturnPacket[16];	//v1.6 chnaged from 8 to 16bytes

	CUsbCode();
	CUSBHIDIO	CUsbHid;		// The usbhid class	
	int		Scan(int, DeviceNameStruct[], int);
	int		GetNthDevice( int Type, int NthDevice, LPSTR DeviceName);
	int		GetNthDeviceDelcom( int Type, int NthDevice, LPSTR DeviceName);
	int		GetDeviceCount(int Type);
	int		GetDeviceCountDelcom(int Type);
	HANDLE	Open( char* );
	int		Close( HANDLE );
	int		SendPacket( HANDLE, pPacketStruct, pPacketStruct ); 
	int		SendHidPacket(HANDLE, pHIDPacketStruct, DWORD len);
	int		IsHandleOpen( HANDLE hDevice);
	HANDLE	IsDeviceOpen(char * Name);
	


	DWORD ReadRS232(HANDLE hUsb, LPSTR DataExt);
	DWORD WriteRS232(HANDLE hUsb, DWORD Length, LPSTR DataExt);
	DWORD RS232Ctrl(HANDLE hUsb, DWORD Mode, DWORD Value);
	DWORD SelReadI2C(HANDLE, BYTE SetAddCmd, BYTE Address, BYTE ReadCmd, BYTE Length, LPSTR DataExt);
	DWORD SelReadI2C16Bit(HANDLE hUsb, BYTE SetAddCmd, WORD Address, BYTE ReadCmd, BYTE Length, LPSTR DataExt);
	DWORD ReadI2C(HANDLE, BYTE, BYTE, LPSTR);
	DWORD WriteI2C(HANDLE, BYTE, BYTE, LPSTR);
	DWORD ReadI2CEEPROM( HANDLE hUsb, DWORD Address, DWORD Length, BYTE CtrlCode,  LPSTR ptr);
	DWORD WriteI2CEEPROM(HANDLE hUsb, DWORD Address, DWORD Length, BYTE CtrlCode,  BYTE WriteDelay, LPSTR ptr);
	DWORD Read64Bit( HANDLE, LPSTR);
	DWORD Write64Bit(HANDLE, LPSTR);
	DWORD ReadPorts(HANDLE, BYTE* , BYTE* );
	DWORD WritePorts(HANDLE hUsb, BYTE Port0, BYTE Port1);
	DWORD WritePin(HANDLE hUsb, BYTE Port, BYTE Pin, BYTE Value);
	DWORD SetupPorts(HANDLE hUsb,BYTE Port,BYTE Mode0,BYTE Mode1);
	
	int EnableAutoConfirm(HANDLE hUsb, int Mode);
	int EnableAutoClear(HANDLE hUsb, int Mode);
	int Buzzer(HANDLE hUsb, uchar Mode, uchar Freq, uchar Repeat, uchar OnTime, uchar OffTime);
	int LoadInitialPhaseDelay(HANDLE hUsb, char Color, char Delay);
	int SyncLeds(HANDLE hUsb);
	int LoadPreScalar(HANDLE hUsb, char PreScalar);
	int LoadLedFreqDuty(HANDLE hUsb, char Color, char Low, char High);
	int GetButtonStatus(HANDLE hUsb );
	int ReadVersionNumber(HANDLE hUsb );
	int ReadSerialNumber(HANDLE hUsb );
	int LED(HANDLE hUsb, int Color, int Mode );
	int LEDPower(HANDLE hUsb, int Color, int Power );

	
	int NumericScanRate( HANDLE hUsb, char ScanRate);
	int NumericInteger(HANDLE hUsb, DWORD Number, DWORD Base, DWORD Radix);
	int NumericSetup( HANDLE hUsb, char Digits);
	int NumericRaw( HANDLE hUsb, LPSTR Str);
	int NumericMode(HANDLE hUsb, char Mode, char Rate);

	// SPI Commands 
	int SPIWrite( HANDLE hUsb, UINT ClockCount, LPSTR pData);
	int SPISetClock( HANDLE hUsb, UINT ClockPeriod );
	int SPIRead(HANDLE hUsb,  LPSTR pData);
	int SPIWr8Read64( HANDLE hUsb, UINT WrData, UINT ReadDataSize, LPSTR pData);


};




// --------------------------------------------------------- //
// EOF
// --------------------------------------------------------- //
